@@ -218,6 +218,11 @@ function switch(){
                       NAMESPACE_ARGUMENT=$2
                       shift
                       ;;
+                  namespace)
+                      NAMESPACE=$1
+                      NAMESPACE_ARGUMENT=$2
+                      shift
+                      ;;
                   history)
                       HISTORY=$1
                       shift
@@ -575,7 +580,7 @@ function setKubeconfigEnvironmentVariable() {
      then
         switchTmpDirectory="$HOME/.kube/.switch_tmp/config"
         if [[ $KUBECONFIG == *"$switchTmpDirectory"* ]]; then
-          rm $KUBECONFIG
+          rm -f $KUBECONFIG
         fi
      fi
 
